﻿/**
 * The purpose of this class is to bring a better control of the framerate
 * @author SamsamTS
 */
import com.samsamts.Events.Event;
import com.samsamts.Events.EventArgs;
import com.samsamts.AnimationCtrl.AnimationEventArgs;

class com.samsamts.AnimationCtrl.Animation
{
	//{ Author properties
	public static var className:String = "Animation";
	public static var classPackage:String = "com.samsamts.Animation";
	public static var version:String = "1.0.0";
	public static var author:String = "SamsamTS";
	public static var link:String = "http://www.samsamts.com/blog/";
	//}
	
	//{ Events
	/**
	* This event isn't the orginal onEnterFrame event from MovieClip class.
	* It use a setInterval with a time check to determine if some frames were lost.
	* Arguments send :
	* - sender : Object : Object who send the event.
	* - arg : AnimationEventArgs : contains a field informing about the number of frame lost.
	*/
	public static var OnEnterFrame:Event = new Event();
	/**
	* Do the same as OnEnterFrame event except that it is only triggered when frames are lost.
	* Arguments are the same too.
	*/
	public static var OnFrameLost:Event = new Event();
	//}
	
	//{ Public
	/**
	* I recommend the use of this function with the true framerate of the movie for better results.
	* You MUST use this function at least once.
	* @param	frameRate the framerate of your animation
	*/
	public static function SetFrameRate(frameRate:Number):Void
	{
		clearInterval($instance.m_intervalId);
		$instance.m_frameRate = frameRate;
		$instance.m_elapsedFrames = 0;
		$instance.m_startTime = getTimer();
		$instance.m_intervalId = setInterval($instance, "OnInterval", Math.round(1000/$instance.m_frameRate));
	}
	public static function GetFrameRate():Number
	{
		return $instance.m_frameRate;
	}
	/**
	* Pause the animation
	* OnEnterframe event is no longer triggered
	*/
	public static function Pause():Void
	{
		clearInterval($instance.m_intervalId);
	}
	/**
	* Resume the animation
	*/
	public static function Resume():Void
	{
		$instance.m_startTime = getTimer();
		$instance.m_elapsedFrames = 0;
		$instance.m_intervalId = setInterval($instance, "OnInterval", Math.round(1000/$instance.m_frameRate));
	}
	public static function GetRemainingTime():Number
	{
		return $instance.m_remainingTime;
	}
	public function toString():String
	{
		return "[Animation]";
	}
	//}
	
	//{ Private
	private static var $instance:Animation = new Animation();
	
	private var m_frameRate:Number;
	private var m_intervalId:Number;
	private var m_startTime:Number;
	private var m_elapsedFrames:Number;
	private var m_remainingTime:Number;

	private function Animation()
	{
		m_frameRate = 25;
		m_startTime = getTimer();
		m_elapsedFrames = 0;
		//m_intervalId = setInterval(this, "OnInterval", Math.round(1000/m_frameRate));
	}
	
	private function OnInterval():Void
	{
		m_elapsedFrames++;
		var frameTime:Number	= 1000/m_frameRate;
		var realTime:Number		= getTimer()-m_startTime;
		var neededTime:Number	= frameTime*m_elapsedFrames;
		var deltaTime:Number	= realTime-neededTime;

		clearInterval(m_intervalId);
		m_remainingTime = Math.floor(frameTime-deltaTime%frameTime);
		
		m_intervalId = setInterval(this, "OnInterval", m_remainingTime);

		var start:Number = getTimer();
		var framesLost:Number = Math.floor(realTime/frameTime)-m_elapsedFrames;
		OnEnterFrame.Trigger(this, new AnimationEventArgs(framesLost));
		if(framesLost > 0)
			OnFrameLost.Trigger(this, new AnimationEventArgs(framesLost));
			
		m_elapsedFrames += framesLost;

		updateAfterEvent();
		m_remainingTime -= getTimer()-start;
		OnAfterFrame.Trigger(this, new EventArgs());
	}
	public static var OnAfterFrame:Event = new Event();
	//}
}