/**
 * The main pupose of this class is to provide a better control of an animation.
 * For example, when an animation is played, if frames are lost the class compensates automatically
 * @author SamsamTS
 * @see Animation
 */
 
import com.samsamts.Events.EventWrapper;
import com.samsamts.Events.EventHandler;
import com.samsamts.AnimationCtrl.Animation;
import com.samsamts.AnimationCtrl.AnimationEventArgs;

class com.samsamts.AnimationCtrl.AnimationClip extends MovieClip
{
	//{ Author properties
	public static var className:String = "AnimationClip";
	public static var classPackage:String = "com.samsamts";
	public static var version:String = "1.0.0";
	public static var author:String = "SamsamTS";
	public static var link:String = "http://www.samsamts.com/blog/";
	//}
	
	//{ Events
	public var OnData:EventWrapper;
	public var OnDragOut:EventWrapper;
	public var OnDragOver:EventWrapper;
	public var OnKillFocus:EventWrapper;
	public var OnPress:EventWrapper;
	public var OnRelease:EventWrapper;
	public var OnReleaseOutside:EventWrapper;
	public var OnRollOut:EventWrapper;
	public var OnRollOver:EventWrapper;
	public var OnSetFocus:EventWrapper;
	public var OnUnload:EventWrapper;
	//}
	
	//{ Static
	/**
	* Convert a MovieClip to an AnimationClip
	* @param	mc The MovieClip to convert
	* @param	deep Optional. If set to true then all MovieClip in mc will be converted too.
	* @return	Reference to rhe converted MovieClip
	*/
	public static function ConvertMovieClip(mc:MovieClip, deep:Boolean):AnimationClip
	{
		mc.__proto__ = AnimationClip.prototype;
		Function(AnimationClip).apply(mc);
		if(deep)
		{
			for(var submc:String in mc)
			{
				if(mc[submc] instanceof MovieClip)
					ConvertMovieClip(mc[submc]);
			}
		}
		return AnimationClip(mc);
	}
	//}

	//{ Public
	public function createEmptyAnimationClip(name:String, depth:Number):AnimationClip
	{
		var mc:MovieClip = super.createEmptyMovieClip.apply(this, arguments);
		return ConvertMovieClip(mc);
	}
	
	public function attachAnimationClip(id:String, name:String, depth:Number, initClass:Function):AnimationClip
	{
		arguments.splice(3,1);
		var mc:MovieClip = super.attachMovie.apply(this, arguments);
		if(!initClass)
			initClass = AnimationClip;
		mc.__proto__ = initClass.prototype;
		initClass.apply(mc);
		for(var submc:String in mc)
		{
			if(mc[submc] instanceof MovieClip)
				ConvertMovieClip(mc[submc]);
		}
		return AnimationClip(mc);
	}
	//}
	
	//{ Overrides
	public function createTextField(instanceName:String, depth:Number, x:Number, y:Number, width:Number, height:Number):TextField
	{
		super.createTextField.apply(this, arguments);
		return this[instanceName];
	}

	public function stop():Void
	{
		if(m_playing)
		{
			m_playing = false;
			Animation.OnEnterFrame.Remove(new EventHandler(this, OnEnterFrameAnimation));
		}
	}

	public function play():Void
	{
		if(!m_playing)
		{
			Animation.OnEnterFrame.Add(new EventHandler(this, OnEnterFrameAnimation));
			m_playing = true;
		}
	}

	public function gotoAndStop(frame:Object):Void
	{
		stop();
		super.gotoAndStop.apply(this, arguments);
	}

	public function gotoAndPlay(frame:Object):Void
	{
		super.gotoAndStop.apply(this, arguments);
		play();
	}

	public function nextFrame():Void
	{
		stop();
		super.nextFrame.apply(this);
	}
	
	public function toString():String
	{
		return "[AnimationClip : "+_name+"]";
	}
	//}
	
	//{ Private
	private var m_playing:Boolean;
	private var onEnterFrame:Function;
	private var onKeyDown:Function;
	private var onKeyUp:Function;
	private var onMouseDown:Function;
	private var onMouseMove:Function;
	private var onMouseUp:Function;

	private function AnimationClip()
	{
		OnData				= new EventWrapper(this, "onData");
		OnDragOut			= new EventWrapper(this, "onDragOut");
		OnDragOver			= new EventWrapper(this, "onDragOver");
		OnKillFocus			= new EventWrapper(this, "onKillFocus");
		OnPress				= new EventWrapper(this, "onPress");
		OnRelease			= new EventWrapper(this, "onRelease");
		OnReleaseOutside	= new EventWrapper(this, "onReleaseOutSide");
		OnRollOut			= new EventWrapper(this, "onRollOut");
		OnRollOver			= new EventWrapper(this, "onRollOver");
		OnSetFocus			= new EventWrapper(this, "onSetFocus");
		OnUnload			= new EventWrapper(this, "onUnload");

		if(_totalframes > 1)
		{
			super.stop.apply(this);
			Animation.OnEnterFrame.Add(new EventHandler(this, OnEnterFrameAnimation));
			m_playing = true;
		}
		
	}

	private function OnEnterFrameAnimation(sender:Object, arg:AnimationEventArgs):Void
	{
		var nb:Number = arg.FramesLost +1;
		while(nb-- > 0 && m_playing)
		{
			if(_currentframe == _totalframes)
				super.gotoAndStop.apply(this, [1]);
			else
				super.nextFrame.apply(this);
		}
	}
	//}
}